import 'package:flutter/material.dart';
import 'package:furman_now/src/services/events/event.dart';
import 'package:furman_now/src/services/events/events_service.dart';
import 'package:furman_now/src/widgets/text_with_icon.dart';
import 'package:intl/intl.dart';

void showEventsModal(BuildContext context, Event event) {
  showModalBottomSheet(
    context: context,
    isScrollControlled: true,
    backgroundColor: Colors.transparent,
    builder: (context) {
      return _EventModal(event: event);
    },
  );
}

class _EventModal extends StatefulWidget {
  final Event event;

  const _EventModal({
    required this.event,
    Key? key,
  }) : super(key: key);

  @override
  State<_EventModal> createState() => _EventModalState();
}

class _EventModalState extends State<_EventModal> {
  var childSize = Size.zero;
  BoxConstraints? _parentConstraints;
  final _key = GlobalKey();

  static const _maxDialogHeight = 0.8;
  
  var _maxChildHeight = _maxDialogHeight;

  void updateMaxHeight() {
    if (_parentConstraints != null) {
      var listHeight = _key.currentContext?.size?.height;
      var parentHeight = _parentConstraints?.maxHeight;
      if (listHeight != null && parentHeight != null) {
        var maxHeight = (listHeight + 70) / parentHeight;
        setState(() {
          _maxChildHeight = (maxHeight < _maxDialogHeight)
              ? maxHeight : _maxDialogHeight;
        });
      }
    }
  }

  @override
  void initState() {
    super.initState();
    WidgetsBinding.instance
      .addPostFrameCallback((_) => updateMaxHeight());
  }

  @override
  Widget build(BuildContext context) {
    return GestureDetector(
      onTap: () => Navigator.of(context).pop(),
      child: Container(
        color: const Color.fromRGBO(0, 0, 0, 0.001),
        child: LayoutBuilder(
          builder: (BuildContext context, BoxConstraints constraints) {
            _parentConstraints = constraints;
            return GestureDetector(
              onTap: () {},
              child: DraggableScrollableSheet(
                  initialChildSize: _maxChildHeight,
                  minChildSize: 0.1,
                  maxChildSize: _maxChildHeight,
                  builder: (_, controller) {
                    return Container(
                      color: Colors.grey.shade100,
                      child: Column(
                        children: [
                          const Align(
                            alignment: Alignment.topRight,
                            child: CloseButton(),
                          ),
                          Flexible(
                            child: NotificationListener
                              <OverscrollIndicatorNotification>(
                              onNotification:
                                (OverscrollIndicatorNotification overscroll) {
                                  overscroll.disallowIndicator();
                                  return true;
                                },
                              child: _EventModalContent(
                                key: _key,
                                controller: controller,
                                event: widget.event,
                              ),
                            ),
                          ),
                        ],
                      ),
                    );
                  }
              ),
            );
          }
        ),
      ),
    );
  }
}

class _EventModalContent extends StatelessWidget {
  final ScrollController controller;
  final Event event;

  const _EventModalContent({
    required this.controller,
    required this.event,
    Key? key,
  }) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return ListView(
      shrinkWrap: true,
      padding: const EdgeInsets.only(left: 40, right: 40, bottom: 40),
      controller: controller,
      children: <Widget>[
        // Title
        Text(
          event.title,
          style: Theme.of(context).textTheme.headlineLarge,
        ),
        const SizedBox(height: 10),
        // Location
        TextWithIcon(
            icon: Icons.place_outlined,
            text: event.location,
        ),
        const SizedBox(height: 5),
        // Additional info for CLP events
        if (event is ClpEvent) ...(() {
          final clpEvent = event as ClpEvent;
          return [
            // Organization
            TextWithIcon(
                icon: Icons.group_outlined,
                text: clpEvent.organization
            ),
            const SizedBox(height: 5),
            // Time
            TextWithIcon(
                icon: Icons.access_time_outlined,
                text:
                  "${DateFormat("EEEE, MMMM d").format(clpEvent.startTime)}"
                  "\n"
                  "${DateFormat.jm().format(clpEvent.startTime)} - ${DateFormat.jm().format(clpEvent.endTime)}",
            ),
            const SizedBox(height: 5),
            // Description
            TextWithIcon(
              icon: Icons.notes,
              text: clpEvent.description
            ),
          ];
        })(),
        // Additional info for Athletics Events
        if (event is AthleticsEvent) ...(() {
          final athleticsEvent = event as AthleticsEvent;
          return [
            // Time
            TextWithIcon(
              icon: Icons.access_time_outlined,
              text:
                "${DateFormat("EEEE, MMMM d").format(athleticsEvent.time)}"
                "\n"
                "${DateFormat.jm().format(athleticsEvent.time)}",
            ),
          ];
        })(),
      ],
    );
  }
}
